package com.fastest.pass.home.presentation.ui.components

import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.BoxScope
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.ColumnScope
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.defaultMinSize
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.imePadding
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.wrapContentHeight
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusManager
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.platform.LocalSoftwareKeyboardController
import androidx.compose.ui.platform.SoftwareKeyboardController
import androidx.compose.ui.res.colorResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.fastest.pass.R
import com.fastest.pass.helpers.BasePreferenceHelper
import com.fastest.pass.home.data.model.NewItemFormField

@Composable
fun AddWifiPasswordFormScreen(
    basePreferenceHelper: BasePreferenceHelper,
    onWifiPasswordFormList: (ArrayList<NewItemFormField>, Boolean) -> Unit
) {
    val keyboardController = LocalSoftwareKeyboardController.current
    val focusManager = LocalFocusManager.current
    var wifi_password_title by remember { mutableStateOf("") }
    var wifi_password_ssid by remember { mutableStateOf("") }
    var wifi_password_password by remember { mutableStateOf("") }
    var wifi_password_connection_type by remember { mutableStateOf("") }
    var wifi_password_connection_mode by remember { mutableStateOf("") }
    var wifi_password_authentication by remember { mutableStateOf("") }
    var wifi_password_encryption by remember { mutableStateOf("") }
    var wifi_password_use_802_1X by remember { mutableStateOf("") }
    var wifi_password_fips_mode by remember { mutableStateOf("") }
    var wifi_password_key_type by remember { mutableStateOf("") }
    var wifi_password_protected by remember { mutableStateOf("") }
    var wifi_password_key_index by remember { mutableStateOf("") }
    var wifi_password_notes by remember { mutableStateOf("") }

    Box(
        modifier = Modifier
            .fillMaxSize()
            .padding(horizontal = 30.dp)
            .padding(bottom = 20.dp)
            .background(Color.Transparent)
            .imePadding()
    ) {
        SaveButtonAWPFS(
            buttonText = R.string.save,
            onSaveButtonClick = {
                val wifiPasswordFormList = ArrayList<NewItemFormField>()
                val saltKey = basePreferenceHelper.getCustomSaltKey()
                val secretKey = basePreferenceHelper.getCustomSecretKey()

                val isTitleEmpty = wifi_password_title.isEmpty()

                wifi_password_title = encryptIfNotEmpty(wifi_password_title, secretKey, saltKey)
                wifi_password_ssid = encryptIfNotEmpty(wifi_password_ssid, secretKey, saltKey)
                wifi_password_password = encryptIfNotEmpty(wifi_password_password, secretKey, saltKey)
                wifi_password_connection_type = encryptIfNotEmpty(wifi_password_connection_type, secretKey, saltKey)
                wifi_password_connection_mode = encryptIfNotEmpty(wifi_password_connection_mode, secretKey, saltKey)
                wifi_password_authentication = encryptIfNotEmpty(wifi_password_authentication, secretKey, saltKey)
                wifi_password_encryption = encryptIfNotEmpty(wifi_password_encryption, secretKey, saltKey)
                wifi_password_use_802_1X = encryptIfNotEmpty(wifi_password_use_802_1X, secretKey, saltKey)
                wifi_password_fips_mode = encryptIfNotEmpty(wifi_password_fips_mode, secretKey, saltKey)
                wifi_password_key_type = encryptIfNotEmpty(wifi_password_key_type, secretKey, saltKey)
                wifi_password_protected = encryptIfNotEmpty(wifi_password_protected, secretKey, saltKey)
                wifi_password_key_index = encryptIfNotEmpty(wifi_password_key_index, secretKey, saltKey)
                wifi_password_notes = encryptIfNotEmpty(wifi_password_notes, secretKey, saltKey)

                wifiPasswordFormList.add(NewItemFormField("wifi-password_title", wifi_password_title))
                wifiPasswordFormList.add(NewItemFormField("wifi-password_ssid", wifi_password_ssid))
                wifiPasswordFormList.add(NewItemFormField("wifi-password_password", wifi_password_password))
                wifiPasswordFormList.add(NewItemFormField("wifi-password_connection_type", wifi_password_connection_type))
                wifiPasswordFormList.add(NewItemFormField("wifi-password_connection_mode", wifi_password_connection_mode))
                wifiPasswordFormList.add(NewItemFormField("wifi-password_authentication", wifi_password_authentication))
                wifiPasswordFormList.add(NewItemFormField("wifi-password_encryption", wifi_password_encryption))
                wifiPasswordFormList.add(NewItemFormField("wifi-password_use_802.1X", wifi_password_use_802_1X))
                wifiPasswordFormList.add(NewItemFormField("wifi-password_fips_mode", wifi_password_fips_mode))
                wifiPasswordFormList.add(NewItemFormField("wifi-password_key_type", wifi_password_key_type))
                wifiPasswordFormList.add(NewItemFormField("wifi-password_protected", wifi_password_protected))
                wifiPasswordFormList.add(NewItemFormField("wifi-password_key_index", wifi_password_key_index))
                wifiPasswordFormList.add(NewItemFormField("wifi-password_notes", wifi_password_notes))

                onWifiPasswordFormList.invoke(wifiPasswordFormList, isTitleEmpty)
            }
        )

        Column(
            modifier = Modifier
                .fillMaxSize()
                .padding(horizontal = 0.dp)
                .padding(bottom = 80.dp)
                .background(Color.Transparent)
                .verticalScroll(rememberScrollState())
        ) {
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAWPFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.title,
                onText = { wifi_password_title = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAWPFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.ssid,
                onText = { wifi_password_ssid = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAWPFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.password,
                onText = { wifi_password_password = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAWPFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.connection_type,
                onText = { wifi_password_connection_type = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAWPFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.connection_mode,
                onText = { wifi_password_connection_mode = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAWPFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.authentication,
                onText = { wifi_password_authentication = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAWPFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.encryption,
                onText = { wifi_password_encryption = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAWPFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.use_802_1x,
                onText = { wifi_password_use_802_1X = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAWPFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.fips_mode,
                onText = { wifi_password_fips_mode = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAWPFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.key_type,
                onText = { wifi_password_key_type = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAWPFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.protected_text,
                onText = { wifi_password_protected = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAWPFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.key_index,
                onText = { wifi_password_key_index = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NotesTextFieldAWFFS(
                keyboardController,
                focusManager,
                onText = { wifi_password_notes = it }
            )
//            Spacer(modifier = Modifier.height(25.dp))
//            SaveButtonAWPFS(buttonText = R.string.save)
        }
    }
}

@Composable
fun ColumnScope.NameTextFieldAWPFS(
    keyboardController: SoftwareKeyboardController?,
    focusManager: FocusManager,
    labelText: Int,
    onText: (String) -> Unit
) {
    var text by remember { mutableStateOf("") }

    TextField(
        value = text,
        onValueChange = {
            text = it
            onText.invoke(text)
        },
        textStyle = MaterialTheme.typography.displayMedium,
        modifier = Modifier
            .align(Alignment.Start)
            .fillMaxWidth()
            .defaultMinSize(minHeight = 60.dp)
            .wrapContentHeight()
            .border(
                1.dp,
                color = colorResource(id = R.color.gray_border_textfield),
                shape = RoundedCornerShape(16.dp)
            )
            .background(color = colorResource(id = R.color.transparent)),
        shape = RoundedCornerShape(16.dp),
//        placeholder = {
//            Text(
//                text = stringResource(id = R.string.enter_email_address),
//                color = colorResource(id = R.color.gray_splash),
//                style = MaterialTheme.typography.displayMedium
//            )
//        },
        label = {
            Text(text = stringResource(id = labelText),
                style = MaterialTheme.typography.titleSmall.copy(
                    color = colorResource(id = R.color.gray_text)
                )
            )
        },
//        leadingIcon = {
//            Image(
//                painter = painterResource(id = R.drawable.profile_circle),
//                contentDescription = "Title Logo",
//                modifier = Modifier
//                    .size(24.dp, 24.dp)
//            )
//        },
        colors = TextFieldDefaults.colors(
            focusedLabelColor = colorResource(id = R.color.gray_splash),
            unfocusedContainerColor = colorResource(id = R.color.transparent),
            focusedContainerColor = colorResource(id = R.color.transparent),
            focusedIndicatorColor = colorResource(id = R.color.transparent),
            disabledIndicatorColor = colorResource(id = R.color.transparent),
            unfocusedIndicatorColor = colorResource(id = R.color.transparent),
            cursorColor = colorResource(id = R.color.gray_splash),
        ),
        keyboardOptions = KeyboardOptions(
            keyboardType = KeyboardType.Text,
            imeAction = ImeAction.Next
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                focusManager.clearFocus()
                keyboardController?.hide()
            }
        ),
    )
}

@Composable
fun ColumnScope.NotesTextFieldAWFFS(
    keyboardController: SoftwareKeyboardController?,
    focusManager: FocusManager,
    onText: (String) -> Unit
) {
    var notesText by remember { mutableStateOf("") }

    TextField(
        value = notesText,
        onValueChange = {
            notesText = it
            onText.invoke(notesText)
        },
        textStyle = MaterialTheme.typography.displayMedium,
        modifier = Modifier
            .align(Alignment.Start)
            .fillMaxWidth()
            .defaultMinSize(minHeight = 60.dp)
            .wrapContentHeight()
            .border(
                1.dp,
                color = colorResource(id = R.color.gray_border_textfield),
                shape = RoundedCornerShape(16.dp)
            )
            .background(color = colorResource(id = R.color.transparent))
        ,
        shape = RoundedCornerShape(16.dp),
//        placeholder = {
//            Text(
//                text = stringResource(id = R.string.enter_email_address),
//                color = colorResource(id = R.color.gray_splash),
//                style = MaterialTheme.typography.displayMedium
//            )
//        },
        label = {
            Text(text = stringResource(R.string.notes),
                style = MaterialTheme.typography.titleSmall.copy(
                    color = colorResource(id = R.color.gray_text)
                )
            )
        },
//        leadingIcon = {
//            Image(
//                painter = painterResource(id = R.drawable.profile_circle),
//                contentDescription = "Title Logo",
//                modifier = Modifier
//                    .size(24.dp, 24.dp)
//            )
//        },
        colors = TextFieldDefaults.colors(
            focusedLabelColor = colorResource(id = R.color.gray_splash),
            unfocusedContainerColor = colorResource(id = R.color.transparent),
            focusedContainerColor = colorResource(id = R.color.transparent),
            focusedIndicatorColor = colorResource(id = R.color.transparent),
            disabledIndicatorColor = colorResource(id = R.color.transparent),
            unfocusedIndicatorColor = colorResource(id = R.color.transparent),
            cursorColor = colorResource(id = R.color.gray_splash),
        ),
        keyboardOptions = KeyboardOptions(
            keyboardType = KeyboardType.Text,
            imeAction = ImeAction.Default
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                focusManager.clearFocus()
                keyboardController?.hide()
            }
        ),
    )
}

@Composable
fun BoxScope.SaveButtonAWPFS(
    buttonText: Int,
    onSaveButtonClick: () -> Unit
) {
    Button(
        modifier = Modifier
            .background(colorResource(id = R.color.transparent))
            .fillMaxWidth()
            .height(60.dp)
            .align(Alignment.BottomCenter)
            .clickable() { },
        onClick = {
            onSaveButtonClick.invoke()
        },
        shape = RoundedCornerShape(15.dp),
//            border = BorderStroke(25.dp, colorResource(id = R.color.black)),
        colors = ButtonDefaults.buttonColors(
            contentColor = colorResource(id = R.color.white),
            containerColor = colorResource(id = R.color.red_login_button),
        ),
    )
    {
        Text(
            text = stringResource(id = buttonText),
            style = MaterialTheme.typography.bodyMedium.copy(
                fontSize = 20.sp,
                color = colorResource(id = R.color.white)
            ),
            textAlign = TextAlign.Center
        )
    }
}